/*  gtklepin - a gtk+ notebook
 *
 *  Copyright (C) 20002  Luc Dufresne <luc@ordiluc.net>
 *                       26, rue des Comices
 *                       59650 Villeneuve d'Ascq
 *                       FRANCE
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  See the file COPYING
 */
 
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#ifdef _WIN32
#include <direct.h>
#else
#include <unistd.h>
#include <time.h>
#include <sys/utsname.h>
#endif /* _WIN32 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/stat.h>
#include <gtk/gtk.h>
#include <zlib.h>
#include <getopt.h>

#ifdef HAVE_LIBZ
#include <zlib.h>
#else
#  define gzopen(String1,String2)           fopen(String1,String2)
#  define gzgets(String1, String2, String3) fgets(String2, String3, String1)
#  define gzeof(String)                     feof(String)
#  define gzread(String1, String2, String3) fread(String2, sizeof (gchar), String3, String1)
#  define gzgetc(String)                    fgetc(String)
#  define gzclose(String)                   fclose(String)
#endif /* HAVE_LIBZ */


#include "interface.h"
#include "support.h"
#include "boulot.h"
#include "misc.h"
#include "options.h"
#include "message.h"
#include "callbacks.h"

/* ar archive */

#define ARMAG   "!<arch>\n"     /* String that begins an archive file.  */
#define ARFMAG  "`\n"           /* String in ar_fmag at end of each header.  */
#define ARUID	"1000  "
#define ARGID	"1000  "
#define ARMODE	"100664  "

struct ar_hdr
  {
    char ar_name[16];           /* Member file name, sometimes / terminated. */
    char ar_date[12];           /* File date, decimal seconds since Epoch.  */
    char ar_uid[6], ar_gid[6];  /* User and group IDs, in ASCII decimal.  */
    char ar_mode[8];            /* File mode, in ASCII octal.  */
    char ar_size[10];           /* File size, in ASCII decimal.  */
    char ar_fmag[2];            /* Always contains ARFMAG.  */
  };

/* end ar */

struct progdata data;
static gint check_write_file (const gchar *file); 

#ifdef DEBUG
static void debug_print (gpointer data, gpointer a);
static void debug (const gchar *str);
#endif

static void version () {
	struct utsname un;
	g_print (""PACKAGE" "VERSION" http://ordiluc.net/"PACKAGE"/\n");
        g_print ("Copyright (c) 2002 Luc Dufresne <luc@ordiluc.net>\n");
	g_print (_("\nGLib Version .......... : %d.%d.%d\n"), glib_major_version, glib_minor_version, glib_micro_version);
	g_print (_("GTK+ Version .......... : %d.%d.%d\n"), gtk_major_version, gtk_minor_version, gtk_micro_version);
	printf (_("Compiled on ........... : "));
	g_print (_(__DATE__ "\n"));
#ifndef _WIN32
	uname (&un);
	g_print (_("Running on ............ : %s %s [%s]\n"), un.sysname, un.release, un.machine);
#endif
}

static void usage () {
        version ();
        g_print ("\n");
        g_print (_("usage: "));
	g_print (PACKAGE);
	g_print (_(" [OPTIONS] [FILE_TO_OPEN]\n"));
        g_print (_("  -h, --help\t\t\tprints this message\n"));
        g_print (_("  -v, --version\t\t\tprints version information\n"));
}

void get_options (int argc, char *argv[]) {
	gint c,opind;
	extern int optind;
	static struct option long_options[] = {
                {"help", 0, 0, 'h'},
                {"version", 0, 0, 'v'},
                {0, 0, 0, 0}
        };

	while(1)
        {
                opind = 0;
                c = getopt_long (argc, argv, "hv", long_options, &opind);
                if (c == -1)
                break;
                switch(c)
                        {
                                case 0:
                                        break;
                                case 'h':
                                        usage();
                                        exit(0);
                                        break;
                                case 'v':
                                        version();
                                        exit(0);
                                        break;
                                default:
                                        usage();
                                        exit(1);
                                        break;
                        }
        }
	data.file_to_open = argv[optind];
}

static struct datastruct * find_data_num (gint page_num) {
	GList *list;

	list = g_list_nth (data.nb_data, page_num);
	return list->data;
}

static struct datastruct * find_data () {
	gint cur_pos;
	GtkNotebook *notebook;

	notebook = (GtkNotebook *) lookup_widget (data.main_window, "notebook");
	cur_pos = gtk_notebook_get_current_page (notebook);
	return find_data_num (cur_pos);
}

static void set_entry (GtkMenuItem *menuitem, gpointer user_data) {
	struct datastruct *nb_data;
	GtkNotebook *notebook;
	GList *list;
	gint pos;
	
	nb_data = (struct datastruct *) user_data;
	notebook = (GtkNotebook *) lookup_widget (data.main_window, "notebook");
	pos = 0;
	list = data.nb_data;
	while (list->data != nb_data) {
		pos++;
		list = list->next;
	}
	gtk_notebook_set_page (notebook, pos);
}

static void foreach_menu_window (gpointer a, gpointer b) {
	GtkWidget *window_menu;
	GtkWidget *new;
	struct datastruct *nb_data = (struct datastruct *) a;
	struct datastruct *data_current = (struct datastruct *) b;
	
	new = gtk_menu_item_new_with_label (nb_data->label->text);
	if (nb_data == data_current) gtk_widget_set_sensitive (new, FALSE);
	
	data.to_destroy = g_list_append (data.to_destroy, new);
	
	gtk_widget_show (new);
	window_menu = lookup_widget (data.main_window, "window_menu");
	gtk_container_add (GTK_CONTAINER (window_menu), new);

	gtk_signal_connect (GTK_OBJECT (new), "activate",
			GTK_SIGNAL_FUNC (set_entry),
			nb_data);
}

void make_menu_window (gint page) {
	GList *list;
	struct datastruct *data_current;

	/* clear the old menu */
	list = g_list_last (data.to_destroy);
	while (data.to_destroy != NULL) {
		gtk_widget_destroy ((GtkWidget *) list->data);
		data.to_destroy = g_list_remove (data.to_destroy, list->data);
		list = list->prev;
	}
	
	/* create the new menu */
	if (g_list_length(data.nb_data) > 0) {
		if (page == -1)	data_current = find_data (); else data_current = find_data_num (page);
		g_list_foreach (data.nb_data, foreach_menu_window, data_current);
	}
}

void switch_page (gint page_num) {
	struct datastruct *data_s;

	if (data.quitting) return;
	data.saved = 0;
	if (g_list_length(data.nb_data) > 1) {
		data_s = find_data_num (page_num);
		gtk_widget_grab_focus ((GtkWidget *) data_s->text);
		make_menu_window (page_num);
	}
}

static void print_line_col (struct datastruct *data_s, gint line, gint col) {
	GString *str;

	str = g_string_new ("");
	g_string_sprintf (str, " %i:%i", line + 1, col + 1);
	gtk_label_set_text ((GtkLabel *) data_s->position_statusbar->label, str->str);
	g_string_free (str, TRUE);
}


static void on_cursor_moved (GtkTextBuffer *buffer, GtkTextIter *arg1, GtkTextMark *arg2, gpointer user_data) {

#ifdef DEBUG	
	debug ("on_cursor_moved");
#endif
	if (arg2 == gtk_text_buffer_get_insert (buffer)) {
		print_line_col (find_data (), gtk_text_iter_get_line (arg1), gtk_text_iter_get_line_offset (arg1));
	}
}

static void set_entry_title (struct datastruct *data_s) {
	gint length;
	gchar *str;
	GString *buf;
	GtkTextBuffer *buffer;
	GtkTextIter start, end, iter;
	GtkTextMark *tm;
	gint line, col;
	
	buffer = gtk_text_view_get_buffer (data_s->text);
	tm = gtk_text_buffer_get_insert (buffer);
	gtk_text_buffer_get_iter_at_mark (buffer, &iter, tm);
	line = gtk_text_iter_get_line (&iter);
	col = gtk_text_iter_get_line_offset (&iter);
	print_line_col (data_s, line, col);
		
//	if (line == 0 && col < 21) {  //buggy
		length = gtk_text_buffer_get_char_count (buffer);
		gtk_text_buffer_get_iter_at_offset (buffer, &start, 0);
		gtk_text_buffer_get_iter_at_offset (buffer, &end, (length > 20) ? 20 : length);

		str = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);
		if (str[0] != '\n') {
			buf = g_string_new (strtok (str,"\n"));
			if (buf->len == 20) g_string_append (buf, "...");
		} else {
			buf = g_string_new ("");
		}
		g_free (str);
		gtk_label_set_text (data_s->label, buf->str);
		g_string_free (buf, TRUE);
//	}
}

static void maj_label (gpointer a, gpointer b) {
	struct datastruct *data_s;
        struct tm *t;
        gchar time_str[1024];
	gchar *time_str_utf8;
	gsize r, w;
	time_t temp;

#ifdef DEBUG	
	debug ("maj_label");
#endif
	data.saved = 0;
	data_s = find_data ();
	set_entry_title (data_s);

	time (&temp);
	if (temp != data_s->modification) {
		data_s->modification = temp;
	        t = localtime (&data_s->modification);
        	strftime (time_str, sizeof (time_str), " %c", t);
		if (NULL != (time_str_utf8 = g_locale_to_utf8 (time_str, sizeof (time_str), &r, &w, NULL))) {
			gtk_label_set_text ((GtkLabel *) data_s->statusbar->label, time_str_utf8);
			g_free (time_str_utf8);
		}
		else	gtk_label_set_text ((GtkLabel *) data_s->statusbar->label, time_str);
	}
}

void init (GtkWidget* window) {
	struct datastruct *data_s;
	GtkNotebook *notebook;
	GtkTextBuffer *buffer;

	data.quitting = FALSE;
	data.main_window = window;
	notebook = (GtkNotebook *) lookup_widget (data.main_window, "notebook");

	data_s = g_malloc (sizeof (struct datastruct));
	data_s->label = (GtkLabel *) lookup_widget ((GtkWidget *) notebook, "label");
	data_s->lower = (GtkButton *) lookup_widget ((GtkWidget *) notebook, "lower_button");
	data_s->raise = (GtkButton *) lookup_widget ((GtkWidget *) notebook, "raise_button");
	data_s->close = (GtkButton *) lookup_widget ((GtkWidget *) notebook, "close_button");
	data_s->text = (GtkTextView *) lookup_widget ((GtkWidget *) notebook, "textview");
	data_s->statusbar = (GtkStatusbar *) lookup_widget ((GtkWidget *) notebook, "statusbar");
	data_s->position_statusbar = (GtkStatusbar *) lookup_widget ((GtkWidget *) notebook, "position_statusbar");

	data.to_destroy = NULL;
	data.nb_data = NULL;
	data.nb_data = g_list_append(data.nb_data, data_s);

	gtk_window_set_title (GTK_WINDOW (data.main_window), ""PACKAGE" "VERSION"");	

	buffer = gtk_text_view_get_buffer (data_s->text);
	g_signal_connect (G_OBJECT (buffer), "changed", G_CALLBACK (maj_label), NULL);
	g_signal_connect_object (G_OBJECT (buffer), "mark_set", G_CALLBACK (on_cursor_moved), NULL, 0);
	
	read_options (data.main_window, data.nb_data);
	open_file ("");
	set_sensitive_button ();
	make_menu_window (-1);
}

static void set_title (const gchar *fichier) {
	GString *title;

	title = g_string_new ("");
	g_string_sprintf (title, ""PACKAGE" "VERSION" - %s" , fichier);
	gtk_window_set_title (GTK_WINDOW (data.main_window), title->str);
	g_string_free (title, TRUE);
}

void clear_text () {
	struct datastruct *data_s;
	GtkTextIter start, end;
	GtkTextBuffer *buffer;

	data_s = find_data ();
	buffer = gtk_text_view_get_buffer (data_s->text);
	gtk_text_buffer_get_iter_at_offset (buffer, &start, 0);
	gtk_text_buffer_get_iter_at_offset (buffer, &end, gtk_text_buffer_get_char_count (buffer));
	gtk_text_buffer_delete (buffer, &start, &end);
}

static gboolean check_file_saved () {
	gint response;

	if (data.saved) return TRUE;
	response = display_ask ((GtkWindow *) data.main_window, _("The file was not saved !\nShould I save it now ?\n"), TRUE);
	switch (response) {
		case GTK_RESPONSE_YES:
			if (save ()) return TRUE;
			return FALSE;
		case GTK_RESPONSE_NO:
			return TRUE;
	}
	return FALSE;
}

struct save_struct {
#ifdef HAVE_LIBZ
	gzFile zfichier;
#endif /* HAVE_LIBZ */
	FILE *fichier;
	gint i, current;
};

static void save_foreach (gpointer data, gpointer a) {
	struct datastruct *nb_data;
	struct save_struct *save_s;
	gint length;
	gchar *str;
	GtkTextBuffer *buffer;
	GtkTextIter start, end;
	
	nb_data = (struct datastruct *) data;
	save_s = (struct save_struct *) a;

	buffer = gtk_text_view_get_buffer (nb_data->text);
	length = gtk_text_buffer_get_char_count (buffer);
	gtk_text_buffer_get_iter_at_offset (buffer, &start, 0);
	gtk_text_buffer_get_iter_at_offset (buffer, &end, length);
	str = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);
	length = strlen (str);

#ifdef HAVE_LIBZ
	if (save_s->zfichier) {
		if (save_s->i != save_s->current) 
			gzprintf (save_s->zfichier, "%-16i%-12li"ARUID""ARGID""ARMODE"%-10i"ARFMAG"", save_s->i, nb_data->modification, length);
		else
			gzprintf (save_s->zfichier, "%-16s%-12li"ARUID""ARGID""ARMODE"%-10i"ARFMAG"", "d", nb_data->modification, length);
		gzprintf (save_s->zfichier, "%s", str);
		if (length % 2) gzprintf (save_s->zfichier, "\n"); /* obligatoire pour avoir une archive lisible avec ar */
	} else {
#endif /* HAVE_LIBZ */
		if (save_s->i != save_s->current) 
			fprintf (save_s->fichier, "%-16i%-12li"ARUID""ARGID""ARMODE"%-10i"ARFMAG"", save_s->i, nb_data->modification, length);
		else
			fprintf (save_s->fichier, "%-16s%-12li"ARUID""ARGID""ARMODE"%-10i"ARFMAG"", "d", nb_data->modification, length);
		fprintf (save_s->fichier, "%s", str);
		if (length % 2) fprintf (save_s->fichier, "\n"); /* obligatoire pour avoir une archive lisible avec ar */
#ifdef HAVE_LIBZ
	}
#endif /* HAVE_LIBZ */

	g_free (str);
        (save_s->i)++;
}

static void do_save (
#ifdef HAVE_LIBZ		
		gzFile zfichier,
#endif /* HAVE_LIBZ */
		FILE *fichier) {
	struct save_struct save_s;
	GtkNotebook *notebook;

#ifdef HAVE_LIBZ	
	if (zfichier) gzprintf (zfichier, ARMAG); else
#endif /* HAVE_LIBZ */
	fprintf (fichier, ARMAG);
	save_s.i = 0;
#ifdef HAVE_LIBZ	
	save_s.zfichier = zfichier;
#endif /* HAVE_LIBZ */
	save_s.fichier = fichier;
	notebook = (GtkNotebook *) lookup_widget (data.main_window, "notebook");
	save_s.current = gtk_notebook_get_current_page (notebook);
	g_list_foreach (data.nb_data, save_foreach, &save_s);
#ifdef HAVE_LIBZ
	if (zfichier) gzclose (zfichier); else 
#endif /* HAVE_LIBZ */
	fclose (fichier);
	data.saved = 1;
}

static GtkWidget* create_file_dialog (const gchar *title) {
	GtkWidget *dialog;

	dialog = gtk_file_selection_new (title);
        gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
	gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_MOUSE);
	gtk_window_set_transient_for ((GtkWindow *) dialog, (GtkWindow *) data.main_window);
	return dialog;
}
	

gint save_as () {
	gchar *str;
	GtkWidget *dialog;
#ifdef HAVE_LIBZ
	gzFile zfichier = NULL;
#endif /* HAVE_LIBZ */
	FILE *fichier = NULL;
	
	dialog = create_file_dialog (_("Save As"));

	if (GTK_RESPONSE_OK == gtk_dialog_run (GTK_DIALOG (dialog))) {
		str = g_strdup (gtk_file_selection_get_filename ((GtkFileSelection *) dialog));
		gtk_widget_destroy (dialog);
		if (!check_write_file (str)) return 0;
#ifdef HAVE_LIBZ
		if (options.Compress) zfichier = gzopen (str, "w9");
		else
#endif /* HAVE_LIBZ */
		fichier = fopen (str, "w");

		if (
#ifdef HAVE_LIBZ
			zfichier == NULL && 
#endif /* HAVE_LIBZ */
			fichier == NULL) {
			display_message ((GtkWindow *) data.main_window, _("Error: Unable to write the file"), GTK_MESSAGE_ERROR);
			return 0;
		}
		do_save (
#ifdef HAVE_LIBZ
			zfichier,
#endif /* HAVE_LIBZ */
			fichier);
		g_string_sprintf(options.LastOpenedFile, "%s", str);
		save_options ();
		set_title (str);
		g_free (str);
		return 1;
	}
	
	gtk_widget_destroy (dialog);
	return 0;
}

static void backup (const gchar *filename) {
	gchar *backup_filename;

	backup_filename = g_strdup_printf ("%s~", filename);
	rename (filename, backup_filename);
	g_free (backup_filename);
}

gint save () {
#ifdef HAVE_LIBZ
	gzFile zfichier = NULL;
#endif /* HAVE_LIBZ */
	FILE *fichier = NULL;

	if (options.LastOpenedFile->str == NULL || 0 == strcmp (options.LastOpenedFile->str, "")) {
		return save_as ();
	}

	if (options.Backup) backup (options.LastOpenedFile->str);
#ifdef HAVE_LIBZ
	if (options.Compress) zfichier = gzopen (options.LastOpenedFile->str, "w9"); else
#endif /* HAVE_LIBZ */
	fichier = fopen (options.LastOpenedFile->str, "w");
	if (
#ifdef HAVE_LIBZ
		zfichier == NULL &&
#endif /* HAVE_LIBZ */
		fichier == NULL) {
		display_message ((GtkWindow *) data.main_window, _("Error: Unable to write the file"), GTK_MESSAGE_ERROR);
		return 0;
	}
	do_save (
#ifdef HAVE_LIBZ
		zfichier,
#endif /* HAVE_LIBZ */
		fichier);
	return 1;
}

gboolean quit () {
	if (!check_file_saved ()) return TRUE;
	save_options ();
	data.quitting = TRUE;
	gtk_main_quit ();
	return FALSE;
}

static void sensitive_on (gpointer data, gpointer a) {
	struct datastruct *nb_data;
	nb_data = (struct datastruct *) data;
	gtk_widget_set_sensitive ((GtkWidget *) nb_data->lower, 1);
	gtk_widget_set_sensitive ((GtkWidget *) nb_data->raise, 1);
	gtk_widget_set_sensitive ((GtkWidget *) nb_data->close, 1);
}

void set_sensitive_button () {
	struct datastruct *data_s;

	data_s = data.nb_data->data;
	if (g_list_length (data.nb_data) == 1) {
		gtk_widget_set_sensitive ((GtkWidget *) data_s->lower, 0);
		gtk_widget_set_sensitive ((GtkWidget *) data_s->raise, 0);
		gtk_widget_set_sensitive ((GtkWidget *) data_s->close, 0);
	} else {
		g_list_foreach (data.nb_data, sensitive_on, NULL);
		gtk_widget_set_sensitive ((GtkWidget *) data_s->raise, 0);
		data_s = g_list_last (data.nb_data) -> data;
		gtk_widget_set_sensitive ((GtkWidget *) data_s->lower, 0);
	}
}

void add_entry (gint pos) {
        GtkNotebook *notebook;
	GtkTextBuffer *buffer;
        GtkWidget *vbox, *hbox, *label, *scrolledwindow, *textview, *position_statusbar, *statusbar, *lower_button, *raise_button, *close_button, *image;
	GtkTooltips *tooltips;	
	struct datastruct *data_s;
	gint cur_pos, true_pos;

        data.saved = 0;

	/* textview */

	vbox = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (vbox);

	scrolledwindow = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_show (scrolledwindow);
	gtk_box_pack_start (GTK_BOX (vbox), scrolledwindow, TRUE, TRUE, 0);
	GTK_WIDGET_UNSET_FLAGS (scrolledwindow, GTK_CAN_FOCUS);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolledwindow), GTK_SHADOW_IN);

	textview = gtk_text_view_new ();
	gtk_widget_show (textview);
	gtk_container_add (GTK_CONTAINER (scrolledwindow), textview);

	hbox = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox);
	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
	
	position_statusbar = gtk_statusbar_new ();
	gtk_widget_show (position_statusbar);
	gtk_box_pack_start (GTK_BOX (hbox), position_statusbar, FALSE, TRUE, 0);
	gtk_widget_set_usize (position_statusbar, 75, -2);
	gtk_statusbar_set_has_resize_grip (GTK_STATUSBAR (position_statusbar), FALSE);
	
	statusbar = gtk_statusbar_new ();
	gtk_widget_show (statusbar);
	gtk_box_pack_start (GTK_BOX (hbox), statusbar, TRUE, TRUE, 0);

	/* label */

	tooltips = gtk_tooltips_new ();
	
	hbox = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox);

	label = gtk_label_new ("");
	gtk_widget_show (label);
	gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);

	lower_button = gtk_button_new ();
	gtk_widget_show (lower_button);
	gtk_box_pack_start (GTK_BOX (hbox), lower_button, FALSE, FALSE, 0);
	gtk_widget_set_usize (lower_button, 20, 20);
	GTK_WIDGET_UNSET_FLAGS (lower_button, GTK_CAN_FOCUS);
	gtk_tooltips_set_tip (tooltips, lower_button, _("Lower Entry"), NULL);
	gtk_button_set_relief (GTK_BUTTON (lower_button), GTK_RELIEF_NONE);

	image = gtk_image_new_from_stock ("gtk-go-down", GTK_ICON_SIZE_MENU);
	gtk_widget_show (image);
	gtk_container_add (GTK_CONTAINER (lower_button), image);

	raise_button = gtk_button_new ();
	gtk_widget_show (raise_button);
	gtk_box_pack_start (GTK_BOX (hbox), raise_button, FALSE, FALSE, 0);
	gtk_widget_set_usize (raise_button, 20, 20);
	GTK_WIDGET_UNSET_FLAGS (raise_button, GTK_CAN_FOCUS);
	gtk_tooltips_set_tip (tooltips, raise_button, _("Raise Entry"), NULL);
	gtk_button_set_relief (GTK_BUTTON (raise_button), GTK_RELIEF_NONE);

	image = gtk_image_new_from_stock ("gtk-go-up", GTK_ICON_SIZE_MENU);
	gtk_widget_show (image);
	gtk_container_add (GTK_CONTAINER (raise_button), image);

	close_button = gtk_button_new ();
	gtk_widget_show (close_button);
	gtk_box_pack_start (GTK_BOX (hbox), close_button, FALSE, FALSE, 0);
	gtk_widget_set_usize (close_button, 20, 20);
	GTK_WIDGET_UNSET_FLAGS (close_button, GTK_CAN_FOCUS);
	gtk_tooltips_set_tip (tooltips, close_button, _("Close Entry"), NULL);
	gtk_button_set_relief (GTK_BUTTON (close_button), GTK_RELIEF_NONE);

	image = gtk_image_new_from_stock ("gtk-close", GTK_ICON_SIZE_MENU);
	gtk_widget_show (image);
	gtk_container_add (GTK_CONTAINER (close_button), image);
	
	data_s = g_malloc (sizeof (struct datastruct));
        data_s->label = (GtkLabel *) label;
        data_s->lower = (GtkButton *) lower_button;
        data_s->raise = (GtkButton *) raise_button;
        data_s->close = (GtkButton *) close_button;
        data_s->text = (GtkTextView *) textview;
        data_s->statusbar = (GtkStatusbar *) statusbar;
	data_s->position_statusbar = (GtkStatusbar *) position_statusbar;

        gtk_signal_connect (GTK_OBJECT (lower_button), "clicked",
                        GTK_SIGNAL_FUNC (on_lower_button_clicked),
                        NULL);
        gtk_signal_connect (GTK_OBJECT (raise_button), "clicked",
                        GTK_SIGNAL_FUNC (on_raise_button_clicked),
                        NULL);
        gtk_signal_connect (GTK_OBJECT (close_button), "clicked",
                        GTK_SIGNAL_FUNC (on_close_button_clicked),
                        NULL);

	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (textview));
	g_signal_connect (G_OBJECT (buffer), "changed", G_CALLBACK (maj_label), NULL);
	g_signal_connect_object (G_OBJECT (buffer), "mark_set", G_CALLBACK (on_cursor_moved), NULL, 0);

        notebook = (GtkNotebook *) lookup_widget (data.main_window, "notebook");
	if (pos == 0) {
		data.nb_data = g_list_append (data.nb_data, data_s);
		
		gtk_notebook_append_page (notebook, vbox, hbox);
		gtk_notebook_set_page (notebook, g_list_length(data.nb_data) - 1);
	} else {
		cur_pos = gtk_notebook_get_current_page (notebook);
		true_pos = 0;
		switch (pos) {
			case -1:
				true_pos = cur_pos;
				break;
			case 1:
				true_pos = cur_pos +1;
				break;
		}
	
		data.nb_data = g_list_insert (data.nb_data, data_s, true_pos);

		gtk_notebook_insert_page (notebook, vbox, hbox, true_pos);
		gtk_notebook_set_page (notebook, true_pos);
	}
	gtk_widget_grab_focus (textview);
	make_menu_window (-1);
}

static void delete_entry_nb (gint i) {
	GtkNotebook *notebook;
	GList *list;
	
	list = g_list_nth (data.nb_data, i);
#ifdef DEBUG
	g_print ("\n\n\n  ### to delete (pos %i) ###\n", i);
	debug_print (list->data, NULL);
#endif	
	g_free (list->data);
	data.nb_data = g_list_remove (data.nb_data, list->data);
	notebook = (GtkNotebook *) lookup_widget (data.main_window, "notebook");
	gtk_notebook_remove_page (notebook, i);
	data.saved = 0;
	make_menu_window (-1);
#ifdef DEBUG
	debug ("delete_entry");
#endif

}

static void delete_entry_nc () {
	gint cur_pos;
	GtkNotebook *notebook;	

	notebook = (GtkNotebook *) lookup_widget (data.main_window, "notebook");
	cur_pos = gtk_notebook_get_current_page (notebook);
	delete_entry_nb (cur_pos);
}

void delete_entry () {
	if (g_list_length(data.nb_data) == 1) return;
	delete_entry_nc ();
}

void close_entry (GtkButton *button_pressed) {
	gint i;
	GList *list;
	struct datastruct *data_s;

	list = data.nb_data;
	data_s = (struct datastruct *) list->data;
	i = 0;
	while (data_s->close != button_pressed) {
		i++;
		list = list->next;
		data_s = (struct datastruct *) list->data;
	}
	delete_entry_nb (i);
}

void line_wrap_callback (gboolean wrap) {
	line_wrap (wrap, data.nb_data);
}

static void move_entry_nb (gint entry, gint mov) {
	GtkNotebook *notebook;
	GList *list;
	struct datastruct *data_s;
	
	notebook = (GtkNotebook *) lookup_widget (data.main_window, "notebook");
	gtk_notebook_reorder_child (notebook, gtk_notebook_get_nth_page (notebook, entry), entry + mov);

	list = g_list_nth(data.nb_data, entry);
	data_s = list->data;
	data.nb_data = g_list_remove (data.nb_data, data_s);
#ifdef DEBUG
	g_print ("\n\n\n  ### to move (pos %i -> %i): ###\n", entry, entry + mov);
	debug_print (data_s, NULL);
#endif	
	data.nb_data = g_list_insert (data.nb_data, data_s, entry + mov);
#ifdef DEBUG
	debug ("move_entry");
#endif
	data.saved = 0;
	make_menu_window (-1);
}

void move_entry (gint i) {
	GtkNotebook *notebook;
	gint cur_pos;

	notebook = (GtkNotebook *) lookup_widget (data.main_window, "notebook");
	cur_pos = gtk_notebook_get_current_page (notebook);
	if (cur_pos + i == -1) return;
	move_entry_nb (cur_pos, i);
}

static gint find_entry_to_move (GtkButton *button_pressed, gint mov) {
	gint i;
	GList *list;
	struct datastruct *data_s;

	list = data.nb_data;
	data_s = (struct datastruct *) list->data;
	i = 0;
	for (;;) {
		switch (mov) {
			case 1:
				if (data_s->lower == button_pressed)
					return i;
			case -1:
				if (data_s->raise == button_pressed)
					return i;
		}
		i++;
		list = list->next;
		data_s = (struct datastruct *) list->data;
	}
	return i;
}

void move_callback (GtkButton *button_pressed, gint mov) {
	move_entry_nb (find_entry_to_move (button_pressed, mov), mov);
}

void import_text_file () {
	gchar *str;
	FILE *fichier;
	GString *buf;
	GtkTextBuffer *buffer;
	GtkTextIter iter;
	struct datastruct *data_s;
	GtkWidget *dialog;

	dialog = create_file_dialog (_("Import Text File"));

	if (GTK_RESPONSE_OK == gtk_dialog_run (GTK_DIALOG (dialog))) {
		str = g_strdup (gtk_file_selection_get_filename ((GtkFileSelection *) dialog));
		gtk_widget_destroy (dialog);
		fichier = fopen (str, "r");
		g_free (str);
		if (fichier == NULL) {
			display_message ((GtkWindow *) data.main_window, _("Error: Unable to read this file"), GTK_MESSAGE_ERROR);
			return;
		}

		add_entry (0);
		data_s = find_data ();
		buffer = gtk_text_view_get_buffer (data_s->text);
		gtk_text_buffer_get_iter_at_offset (buffer, &iter, 0);
		
		while (!feof(fichier)) {
	                buf = getline (fichier);
			gtk_text_buffer_insert (buffer, &iter, buf->str, buf->len);
			g_string_free(buf,TRUE);
		}
		fclose (fichier);

	} else gtk_widget_destroy (dialog);
}

static gint check_write_file (const gchar *file) {
	struct stat buf;

	if (-1 == stat (file, &buf)) {
		return 1;
	} else {
#ifdef _WIN32
		if (S_IFDIR & buf.st_mode) {
#else
		if S_ISDIR(buf.st_mode) {
#endif
			display_message ((GtkWindow *) data.main_window, _("Error: This is a directory"), GTK_MESSAGE_ERROR);
			return 0;
		}
		if (GTK_RESPONSE_YES == display_ask ((GtkWindow *) data.main_window, _("This file already exists\nShould I overwrite it ?"), FALSE)) return 1;
	}
	return 0;
}

void export_current_entry () {
	struct datastruct *data_s;
	gchar *str;
	FILE *fichier;
	GtkWidget *dialog;
	GtkTextBuffer *buffer;
	gint length;
	GtkTextIter start, end;

	dialog = create_file_dialog (_("Export Current Entry"));

	if (GTK_RESPONSE_OK == gtk_dialog_run (GTK_DIALOG (dialog))) {
		str = g_strdup (gtk_file_selection_get_filename ((GtkFileSelection *) dialog));
		gtk_widget_destroy (dialog);
		if (!check_write_file (str)) return;
		fichier = fopen (str, "w");
		g_free (str);
		if (fichier == NULL) {
			display_message ((GtkWindow *) data.main_window, _("Error: Unable to write the file"), GTK_MESSAGE_ERROR);
			return;
		}
		data_s = find_data ();
		buffer = gtk_text_view_get_buffer (data_s->text);
		length = gtk_text_buffer_get_char_count (buffer);
		gtk_text_buffer_get_iter_at_offset (buffer, &start, 0);
		gtk_text_buffer_get_iter_at_offset (buffer, &end, length);
		str = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);

		fprintf (fichier, "%s", str);
		g_free (str);
		fclose (fichier);
	} else gtk_widget_destroy (dialog);
}


static gint check_directory (const gchar *file) {
	struct stat buf;
	
	if (-1 == stat (file, &buf)) {
		return 1;
	} else {
		display_message ((GtkWindow *) data.main_window, _("Error: Something with that name already exists"), GTK_MESSAGE_ERROR);
		return 0;
	}
	return 0;
}

struct export_struct {
	gchar *dir;
	gint i;
	gboolean error;
};

static void export_foreach (gpointer data, gpointer a) {
	struct datastruct *nb_data;
	struct export_struct *export_s;
	gint length;
	gchar *str;
	FILE *fichier;
	GString *string;
	GtkTextBuffer *buffer;
	GtkTextIter start, end;

	nb_data = (struct datastruct *) data;
	export_s = (struct export_struct *) a;

	string = g_string_new ("");
	g_string_sprintf (string,"%s/%i", export_s->dir, export_s->i);
	fichier = fopen (string->str, "w");
	g_string_free (string, TRUE);
	if (fichier == NULL) {
		export_s->error = 1;
		return;
	}

	buffer = gtk_text_view_get_buffer (nb_data->text);
	length = gtk_text_buffer_get_char_count (buffer);
	gtk_text_buffer_get_iter_at_offset (buffer, &start, 0);
	gtk_text_buffer_get_iter_at_offset (buffer, &end, length);
	str = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);
	fprintf (fichier, "%s", str);

	g_free (str);
	fclose (fichier);	
	(export_s->i)++;	
}

void export_directory () {
	gchar *str;
	struct export_struct export_s;	
	GtkWidget *dialog;
	
	dialog = create_file_dialog (_("Export In a Directory"));
	
	if (GTK_RESPONSE_OK == gtk_dialog_run (GTK_DIALOG (dialog))) {
		str = g_strdup (gtk_file_selection_get_filename ((GtkFileSelection *) dialog));
		gtk_widget_destroy (dialog);
		if (!check_directory (str)) return;
#ifdef _WIN32
		if (-1 == mkdir (str)) {
#else
		if (-1 == mkdir (str,S_IRWXU)) {
#endif
			g_free (str);
			display_message ((GtkWindow *) data.main_window, _("Error: Unable to create this directory"), GTK_MESSAGE_ERROR);
			return;
		}
		export_s.i = 0;
		export_s.dir = str;
		export_s.error = 0;
		g_list_foreach (data.nb_data, export_foreach, &export_s);
		if (export_s.error) display_message ((GtkWindow *) data.main_window, _("Error: Could not write everything"), GTK_MESSAGE_ERROR);
		g_free (str);
	} else gtk_widget_destroy (dialog);
}

static void create_new_nc () {
	/* delete everything */
	while (g_list_length(data.nb_data) != 0) delete_entry_nc ();

	/* create a new entry */
	add_entry (0);

	data.saved = 1;
	make_menu_window (-1);
	gtk_window_set_title (GTK_WINDOW (data.main_window), ""PACKAGE" "VERSION"");
	g_string_assign (options.LastOpenedFile, "");
	save_options ();
}

void create_new () {
	if (!check_file_saved ()) return;
	create_new_nc ();
}

void open_file (gchar *fichier_a_ouvrir) {
#ifdef HAVE_LIBZ
	gzFile fichier;
#else
	FILE *fichier;
#endif /* HAVE_LIBZ */
	gchar buf[sizeof(struct ar_hdr) + 1];
	gchar tmp[sizeof(struct ar_hdr) + 1];
	gchar tmp2[sizeof(ARFMAG)];
	gint size, i, page;
	gchar *str;
	struct datastruct *data_s;
	GtkNotebook *notebook;
	struct tm *t;
	gchar time_str[1024];
	gchar *time_str_utf8;
	time_t date;
	gchar *fichier_a_ouvrir_reel;
	GString *err;
	GtkTextBuffer *buffer;
	GtkTextIter iter;
	gsize r, w;
	
	if (0 == strcmp (fichier_a_ouvrir, "")) {   /* au dmarrage du programme */
		if (NULL != data.file_to_open  &&
				0 != strcmp (data.file_to_open, "")) {
			fichier_a_ouvrir_reel = data.file_to_open;
#ifdef DEBUG
			g_print ("command line\n");
#endif
		}
		else {
			if (options.OpenLast &&
					options.LastOpenedFile->str != NULL &&
					0 != strcmp (options.LastOpenedFile->str, "")) {
				fichier_a_ouvrir_reel = options.LastOpenedFile->str;
#ifdef DEBUG
				g_print ("last opened file\n");
#endif
			}
			else {
#ifdef DEBUG
				g_print ("nothing to open\n");
#endif
				data.saved = 1;
				return;
			}
		}
	} else {
		/* delete everything */
		while (g_list_length(data.nb_data) != 0) delete_entry_nc ();

		/* create a new entry */
		add_entry (0);

		fichier_a_ouvrir_reel = fichier_a_ouvrir;
	}
#ifdef DEBUG
	g_print ("file to open: --%s--\n", fichier_a_ouvrir_reel);
#endif
	fichier = gzopen (fichier_a_ouvrir_reel, "r");
	if (fichier == NULL) {
		err = g_string_new ("");
		g_string_sprintf (err, _("Error: Unable to read the file: \"%s\""), fichier_a_ouvrir_reel);
		display_message ((GtkWindow *) data.main_window, err->str, GTK_MESSAGE_ERROR);
		g_string_free (err, TRUE);
		data.saved = 1;
		make_menu_window (-1);
		gtk_window_set_title (GTK_WINDOW (data.main_window), ""PACKAGE" "VERSION"");
		g_string_assign (options.LastOpenedFile, "");
		return;		
	}

	if (NULL == gzgets (fichier, buf, sizeof(ARMAG) +1) ||
			0 != strcmp (buf, ARMAG)) {
		err = g_string_new ("");
		g_string_sprintf (err, _("Error: The file \"%s\" is not valid"), fichier_a_ouvrir_reel);
		display_message ((GtkWindow *) data.main_window, err->str, GTK_MESSAGE_ERROR);
		g_string_free (err, TRUE);
		data.saved = 1;
		make_menu_window (-1);
		gtk_window_set_title (GTK_WINDOW (data.main_window), ""PACKAGE" "VERSION"");
		g_string_assign (options.LastOpenedFile, "");
		return;
	}

	i = 0;
	page = 0;
	while (!gzeof(fichier)) {
		if(NULL == gzgets (fichier, buf, sizeof(struct ar_hdr) + 1)) break;

		strncpy (tmp2, &buf[58],2);
		tmp2[2] = '\0';
		if (0 != strcmp (tmp2, ARFMAG)) {
			display_message ((GtkWindow *) data.main_window, _("Error while reading the file\nThe file might not be valid"), GTK_MESSAGE_ERROR);
			create_new_nc ();
			return;
		}
	
		strncpy(tmp, buf, 1);
		tmp[1] = '\0';
		if (0 == strcmp(tmp,"d")) page = i;
		
		strncpy (tmp, &buf[16],12);
		tmp[12] = '\0';
		date = atoi (tmp);

		strncpy (tmp, &buf[48],10);
		tmp[12] = '\0';
		size = atoi (tmp);
		if (i) add_entry (0);

		data_s = find_data ();
		if(size > 0) {
			str = g_malloc (size + 1);
			if (size != gzread(fichier, str, size)) {
				display_message ((GtkWindow *) data.main_window, _("Error while reading the file\nThe file might not be valid"), GTK_MESSAGE_ERROR);
				g_free (str);
				create_new_nc ();
				return;
				}
			buffer = gtk_text_view_get_buffer (data_s->text);
			gtk_text_buffer_get_iter_at_offset (buffer, &iter, 0);
			gtk_text_buffer_insert (buffer, &iter, str, size);

			gtk_text_buffer_get_iter_at_offset (buffer, &iter, 0);
			gtk_text_buffer_place_cursor (buffer, &iter);
			if(size % 2) gzgetc (fichier);
			g_free (str);
		}

		data_s->modification = date;
		t = localtime (&data_s->modification);
	        strftime (time_str, sizeof (time_str), " %c", t);
		if (NULL != (time_str_utf8 = g_locale_to_utf8 (time_str, sizeof (time_str), &r, &w, NULL))) {
			gtk_label_set_text ((GtkLabel *) data_s->statusbar->label, time_str_utf8);
			g_free (time_str_utf8);
		}
		else	gtk_label_set_text ((GtkLabel *) data_s->statusbar->label, time_str);
		
		i++;
	}
	gzclose (fichier);

	notebook = (GtkNotebook *) lookup_widget (data.main_window, "notebook");
	gtk_notebook_set_page(notebook, page);
	make_menu_window (-1);
	data.saved = 1;

	fichier_a_ouvrir_reel = g_strdup (fichier_a_ouvrir_reel);
	g_string_sprintf(options.LastOpenedFile, "%s", fichier_a_ouvrir_reel);
	save_options ();
	set_title (fichier_a_ouvrir_reel);
	g_free (fichier_a_ouvrir_reel);
}

void do_open_file () {
	gchar *str;
	GtkWidget *dialog;

	if (!check_file_saved ()) return;

	dialog = create_file_dialog (_("Open File"));

	if (GTK_RESPONSE_OK == gtk_dialog_run (GTK_DIALOG (dialog))) {
		str = g_strdup (gtk_file_selection_get_filename ((GtkFileSelection *) dialog));
		gtk_widget_destroy (dialog);
		open_file (str);
		g_free (str);
	} else gtk_widget_destroy (dialog);
}

#ifdef DEBUG
static void debug_print (gpointer data, gpointer a) {
	gint length;
	GtkTextBuffer *buffer;
	GtkTextIter start, end;
	struct datastruct *nb_data;
	gchar *str;

	nb_data = (struct datastruct *) data;
	buffer = gtk_text_view_get_buffer (nb_data->text);
	length = gtk_text_buffer_get_char_count (buffer);
	gtk_text_buffer_get_iter_at_offset (buffer, &start, 0);
	gtk_text_buffer_get_iter_at_offset (buffer, &end, length);
	str = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);	

	g_print ("----\n%s\n----\n", str);
	g_free (str);
}

static void debug (const gchar *str) {

	g_print ("\n\n\n  ### DEBUG: %s ###\n",str);
	g_list_foreach (data.nb_data, debug_print, NULL);
}
#endif

