/*  gtklepin - a gtk+ notebook
 *
 *  Copyright (C) 2002  Luc Dufresne <luc@ordiluc.net>
 *                       26, rue des Comices
 *                       59650 Villeneuve d'Ascq
 *                       FRANCE
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  See the file COPYING
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"

#include "options.h"
#include "boulot.h"
#include "message.h"

/*
 *  quit
 */

gboolean
on_main_window_delete_event            (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
  return quit();
}


gboolean
on_main_window_destroy_event           (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
  return FALSE;
}

void
on_quit_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  quit ();
}

void
on_button_quit_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
  quit ();
}

/*
 *  about / preferences
 */

void
on_about_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  open_options_window (1);
}

void
on_button_about_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
  open_options_window (1);
}

void
on_config_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  open_options_window (0);
}

void
on_checkbutton_toolbar_toggled         (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  show_toolbar_toggle (togglebutton);
}

void
on_radiobutton_both_toggled            (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  if (GTK_TOGGLE_BUTTON (togglebutton)->active) toolbar_style (0);
}

void
on_radiobutton_pictures_toggled        (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  if (GTK_TOGGLE_BUTTON (togglebutton)->active) toolbar_style (1);
}

void
on_radiobutton_text_toggled            (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  if (GTK_TOGGLE_BUTTON (togglebutton)->active) toolbar_style (2);
}

void
on_radiobutton_left_toggled            (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  if (GTK_TOGGLE_BUTTON (togglebutton)->active) tabs_pos (0);
}

void
on_radiobutton_right_toggled           (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  if (GTK_TOGGLE_BUTTON (togglebutton)->active) tabs_pos (1);
}

void
on_radiobutton_top_toggled             (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  if (GTK_TOGGLE_BUTTON (togglebutton)->active) tabs_pos (2);
}

void
on_radiobutton_bottom_toggled          (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  if (GTK_TOGGLE_BUTTON (togglebutton)->active) tabs_pos (3);
}

void
on_checkbutton_linewrap_toggled        (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  line_wrap_callback (GTK_TOGGLE_BUTTON (togglebutton)->active);
}

/*
 *  ajouts
 */
void
on_add_entry_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  add_entry (0);
  set_sensitive_button ();
}

void
on_button_new_entry_clicked            (GtkButton       *button,
                                        gpointer         user_data)
{
  add_entry (0);
  set_sensitive_button ();
}


void
on_insert_entry_before_activate        (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  add_entry (-1);
  set_sensitive_button ();
}


void
on_insert_entry_after_activate         (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  add_entry (1);
  set_sensitive_button ();
}

/*
 *  dplacements
 */
void
on_raise_entry_activate                (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  move_entry (-1);
  set_sensitive_button ();
}

void
on_lower_entry_activate                (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  move_entry (1);
  set_sensitive_button ();
}

void
on_raise_button_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
  move_callback (button, -1);
  set_sensitive_button ();
}

void
on_lower_button_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
  move_callback (button, 1);
  set_sensitive_button ();
}

/*
 *  effacer
 */

void
on_delete_entry_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  delete_entry ();
  set_sensitive_button ();
}

void
on_close_button_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
  close_entry (button);
  set_sensitive_button ();
}

void
on_clear_entry_activate                (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  clear_text ();
}

/*
 *  ouvrir
 */

void
on_open_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  do_open_file ();
  set_sensitive_button ();
}

void
on_button_open_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
  do_open_file ();
  set_sensitive_button ();
}

/*
 *  nouveau
 */

void
on_new_activate                        (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  create_new ();
  set_sensitive_button ();
}

void
on_button_new_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
  create_new ();
  set_sensitive_button ();
}

/*
 *  sauvegarder
 */

void
on_save_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}

void
on_save_as_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  save_as ();
}

void
on_button_save_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
  save ();
}

/*
 *  exporter / importer
 */

void
on_import_text_file_activate           (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  import_text_file ();
}

void
on_export_entry_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  export_current_entry ();
}

void
on_export_everything_in_a_directory_activate
                                        (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  export_directory ();
}

/*
 * divers
 */

void
on_notebook_switch_page                (GtkNotebook     *notebook,
                                        GtkNotebookPage *page,
                                        gint             page_num,
                                        gpointer         user_data)
{
  switch_page (page_num);
}


void
on_window_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  make_menu_window (-1);
}

